// bcell_bzip2.cc - Memory Cell Abstraction Interface (Bcell: Byte Mcell)
// bzip2 functions
// GPLed - Stephan T. Lavavej - http://stl.caltech.edu

#include "bcell.hh"

#ifdef BZIP

// Compress our cell using bzip2
Bcell& Bcell::bzip2() {

    unsigned int sourceLen = size;
    unsigned int destLen = size + (size >> 6) + 600;

    char *source = new char[sourceLen];
    char *dest = new char[destLen];

    for (size_t i = 0; i < sourceLen; i++) {
        source[i] = (*this)[i];
    }

    int error = BZ2_bzBuffToBuffCompress(dest, &destLen, source, sourceLen, 9, 0, 0);

    // Upon exit, destLen is the actual size of the compressed buffer.

    if (error != BZ_OK) {
        cout << "ERROR: libbzip reported an error." << endl;
        exit(EXIT_FAILURE);
    }

    Bcell compressed("bzip2 Compressed Bcell", destLen);

    for (size_t i = 0; i < destLen; i++) {
        compressed[i] = dest[i];
    }

    delete[] source;
    delete[] dest;

    *this = compressed;

    return *this;
}

// Decompress our cell using bzip2
Bcell& Bcell::unbzip2(unsigned int originalsize) {

    unsigned int sourceLen = size;
    unsigned int destLen = originalsize;

    char *source = new char[sourceLen];
    char *dest = new char[destLen];

    for (size_t i = 0; i < sourceLen; i++) {
        source[i] = (*this)[i];
    }

    int error = BZ2_bzBuffToBuffDecompress(dest, &destLen, source, sourceLen, 0, 0);

    // Upon exit, destLen is the actual size of the decompressed buffer.

    if (error != BZ_OK) {
        cout << "ERROR: libbzip reported an error." << endl;
        exit(EXIT_FAILURE);
    }

    Bcell decompressed("bzip2 Decompressed Bcell", destLen);

    for (size_t i = 0; i < destLen; i++) {
        decompressed[i] = dest[i];
    }

    delete[] source;
    delete[] dest;

    *this = decompressed;

    return *this;
}

#endif // End of: #ifdef BZIP
