// bcell_file.cc - Memory Cell Abstraction Interface (Bcell: Byte Mcell)
// File functions
// GPLed - Stephan T. Lavavej - http://stl.caltech.edu

#include "bcell.hh"

// PURPOSE: Fill our cell with the entire contents of a file
// ERROR:   Scream and die if our cell is not big enough
// NOTE:    File left rewound
// RETURN:  Our cell
Bcell& Bcell::readFile(FILE *fp) {

    size_t fsize = file_size(fp);

    if (size < fsize) {
        cout << "ERROR: Attempt to overfill cell " << name;
        cout << ", size " << size << " from a file." << endl;
        exit(EXIT_FAILURE);
    }

    rewind(fp);

    size_t bytesread = fread(buf, 1, fsize, fp);

    if (bytesread != fsize || ferror(fp)) { 
        cout << "ERROR: Problem reading file into cell " << name;
        cout << ", size " << size << endl;
        exit(EXIT_FAILURE);
    }

    rewind(fp);
    return *this;
}

// PURPOSE: Write the contents of our cell to a file
// RETURN:  Our cell
Bcell& Bcell::writeFile(FILE *fp) {

    size_t bytessent = fwrite(buf, 1, size, fp);

    if (bytessent != size || ferror(fp)) { 
        cout << "ERROR: Problem writing cell to file. Cell " << name;
        cout << ", size " << size << endl;
        exit(EXIT_FAILURE);
    }

    return *this;
}

// PURPOSE: Construct a cell whose contents are that of a named file
Bcell::Bcell(const char *fname) : Mcell<unsigned char>() {
    FILE *fp = file_open_rb(fname);
    alloc(fname, file_size(fp));
    readFile(fp);
    file_close(fp);
}

// PURPOSE: Write the contents of our cell to a named file
// RETURN:  Our cell
Bcell& Bcell::writeFileName(const char *fname) {
    FILE *fp = file_open_wb(fname);
    writeFile(fp);
    file_close(fp);
    return *this;
}
