// main_bwtcompare.cc - Main function for bwtcompare
// GPLed - Stephan T. Lavavej - http://stl.caltech.edu

#include "bcell.hh"

int main(int argc, char *argv[]) {
    if (argc != 2) {
        cout << "USAGE: bwtcompare infile" << endl;
        exit(EXIT_SUCCESS);
    }

    Bcell cell(argv[1]);
    const Bcell orig = cell;

#ifdef FENWICK_OPTIMIZATION
    cout << "Using Fenwick optimization." << endl;
#else
    cout << "Not using Fenwick optimization." << endl;
#endif

    cout << "Original file size: " << cell.getSize() << endl;
    cell.bwtSarray().mtf().huff();
	cout << "BWT Suffix Array / MTF / Huff: " << cell.getSize() << endl;
    cell.puff().unmtf().unbwtSarray().bwtStree().mtf().huff();
	cout << "BWT Suffix Tree / MTF / Huff: " << cell.getSize() << endl;
    cell.puff().arith();
	cout << "BWT Suffix Tree / MTF / Arith: " << cell.getSize() << endl;
    cell.unarith().unmtf().mtf2().arith();
	cout << "BWT Suffix Tree / MTF-2 / Arith: " << cell.getSize() << endl;
    cell.unarith().srle().arith();
	cout << "BWT Suffix Tree / MTF-2 / SRLE / Arith: " << cell.getSize() << endl;
    cell.unarith().unsrle().srle0().arith();
	cout << "BWT Suffix Tree / MTF-2 / SRLE-0 / Arith: " << cell.getSize() << endl;
    cell.unarith().unsrle0().srle0a().arith();
	cout << "BWT Suffix Tree / MTF-2 / SRLE-0-A / Arith: " << cell.getSize() << endl;
    cell.unarith().unsrle0a().zle().arith();
	cout << "BWT Suffix Tree / MTF-2 / ZLE / Arith: " << cell.getSize() << endl;
    cell.unarith().unzle().zle1().arith();
	cout << "BWT Suffix Tree / MTF-2 / ZLE-1 / Arith: " << cell.getSize() << endl;
    cell.unarith().unzle1().zleWheeler().arith();
	cout << "BWT Suffix Tree / MTF-2 / Wheeler ZLE / Arith: " << cell.getSize() << endl;
    cell.unarith().unzleWheeler().unmtf2().unbwtStree();	

    if (cell == orig) {
        cout << "Success!" << endl;
    } else {
        cout << "Failure!" << endl;
    }
}
